﻿using System.Windows;
using System.Windows.Input;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Data;
using System;

namespace WpfCommands
{
    /// <summary>
    /// Interaction logic for CommandsDemo.xaml
    /// </summary>
    public partial class CommandsDemo
    {
        /// <summary>
        /// Command to sort the Customers
        /// As ou can see the command also hooks to the F3 key of the keyboard
        /// </summary>
        public static RoutedUICommand SortCustomers = new RoutedUICommand("SortCustomers",
                "SortCustomers", typeof(CommandsDemo), new InputGestureCollection(
                    new InputGesture[] { new KeyGesture(Key.F3, ModifierKeys.None, "Sort Customer") }
                    ));

        /// <summary>
        /// Command to delete a specific customer
        /// </summary>
        public static RoutedUICommand DeleteCustomer = new RoutedUICommand("DeleteCustomer",
                "DeleteCustomer", typeof(CommandsDemo));
        
        //data service taht generates sample data
        SampleDataService dataService;

        public CommandsDemo()
        {
            InitializeComponent();
            
            //create an instance of the sample data service
            dataService = new SampleDataService();
            customerList.ItemsSource = dataService.CustomerList;//set the list of customers as ItemsSource
            //This is the important step for the demo
            //Here we create handle the command binding for the whole window
            //so which ever control invokes this command the same handler will handle the command
            CommandBindings.Add(new CommandBinding(SortCustomers, SortCustomersHandler));

            //command binding for the delet customer
            CommandBindings.Add(new CommandBinding(DeleteCustomer, DeleteCustomerHandler, CanExecuteDeleteCustomer));

            //handle the CanExecuteChange to set the text of the delete text block
            DeleteCustomer.CanExecuteChanged += delegate
            {
                bool canExecute = AvalonCommandsHelper.CanExecuteCommandSource(deleteButton);
                deleteText.Text = canExecute ?
                    "Delete selected customer" :
                    "Select customer to delete";
            };
            
        }
        
        //handler for the Sort Customers command.
        //this event handler will be invoked when some element will execute the SortCustomers Command
        private void SortCustomersHandler(object sender, ExecutedRoutedEventArgs e)
        {
            //default the sort if no parameter is passed
            string sortBy = e.Parameter == null ?
                "Name" :
                e.Parameter.ToString();

            //get the view for the list
            ICollectionView view = CollectionViewSource.GetDefaultView(customerList.ItemsSource);
            view.SortDescriptions.Clear();
            //add the new sort description
            view.SortDescriptions.Add(
                new SortDescription(sortBy, ListSortDirection.Ascending));
            view.Refresh();
        }

        //handler for the Can execute chack of the delete customer
        private void CanExecuteDeleteCustomer(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = e.Parameter != null && e.Parameter is Customer; ;
            e.Handled = true;
        }

        //handler for the delete customer command
        private void DeleteCustomerHandler(object sender, ExecutedRoutedEventArgs e)
        {
            Customer selectedCust = (Customer)e.Parameter;
            dataService.CustomerList.Remove(selectedCust);
        }

        //event handler for the selection changed of the combo box
        //This shows how to trigger a command from code
        private void SortByFieldSelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            ICommand command = SortCustomers;
            command.Execute(e.AddedItems[0]);
        }
    }
}
